import time

from netforce.model import Model, fields, get_model
from netforce.access import get_active_company, get_active_user, set_active_user

from . import utils

class Patient(Model):
    _name="clinic.patient"
    _string="Patient"
    _audit_log=True
    _multi_company=True
    _key=['number','branch_id']

    def _get_age(self,ids,context):
        res={}
        year_now=int(time.strftime("%Y"))
        for obj in self.browse(ids):
            age=0
            if obj.birthday:
                year_bd=int(obj.birthday[0:4])
                age=year_now-year_bd
            res[obj.id]=age
        return res # -> {1: 30, 2: 45,.....}
    
    def _get_last_cycle(self,ids,context={}):
        res={}
        for obj in self.browse(ids):
            cycle_id=None
            for vs in obj.visits:
                cycle=vs.cycle_id
                cycle_id=cycle.id
            res[obj.id]=cycle_id
        return res
    
    def _get_hn_no(self,ids,context={}):
        res={}
        for obj in self.browse(ids):
            hn=''.join(x for x in (obj.number or "") if x.isdigit())
            res[obj.id]=hn
        return res

    _fields={
        'type_id': fields.Many2One("clinic.patient.type","Type",search=True,required=True),
        "number": fields.Char("HN Number",required=True,search=True),
        "trt_no": fields.Char("TRT",search=True),
        "hn_no": fields.Char("HN",function="_get_hn_no",store=True),
        "hn": fields.Char("REF/HN",search=False),
        "name": fields.Char("Name",required=True,search=True),
        "reg_date": fields.Date("Register Date",required=False,search=True),
        "birthday": fields.Date("Birthday",required=False,search=True),
        "phone": fields.Char("Phone",required=False,search=True),
        "mobile": fields.Char("Mobile",required=False,search=True),
        "job": fields.Char("Job/Position"),
        "age": fields.Integer("Age", function="_get_age"),
        'image': fields.File("Image"),
        'email': fields.Char("Email"),
        "weight": fields.Float("Weight (kg.)"),
        "height": fields.Float("Height (cm.)"),
        "card_type": fields.Selection([("identification","Identification"),("passport","Passport")],"ID Type"),
        'card_no' : fields.Char("ID"),
        'card_exp' : fields.Date("ID Exp."),
        "app_no": fields.Char("Application No."),
        "salary": fields.Selection([["20000","5,001-20,000"],["50000","20,001-50,000"],["100000","50,001-100,000"],["100001","100,000+"]], "Salary"),
        "addresses": fields.One2Many("address","patient_id","Addresses"),
        "gender": fields.Selection([("male","Male"),("female","Female")],"Gender",search=True),
        "marital_status": fields.Selection([("single","Single"),("marry","Marry"),("divorce","Divorce"),("separated","Saparated"),("widowed","Widowed")],"Marital Status",required=False),
        "nation_id": fields.Many2One("clinic.nation","Nationality"),
        "race_id": fields.Many2One("clinic.race","Race"),
        "grad_id": fields.Many2One("clinic.graduation","Graduation"),
        "smoke": fields.Selection([("never","Never"),("stopped","Stopped"),("smoked","Smoked")],"Smoking"),
        "withdrawal" : fields.Selection([("social_security","Social Security"),("health_insurance","Health Insurance"),("etc","ETC.")],"Right of withdrawal"),
        "first_treatment" : fields.Selection([("hd","HD"),("test","Test")], "First treatment"),
        "first_hemodialysis": fields.Date("First time Hemodialysis",required=False),
        "hemodialysis": fields.Char("First Hemodialysis",required=False),
        "clinic_after": fields.Selection([("small","Small"),("medium","Medium"),("large","Large")],"Clinic Lastime",required=False),
        "clinic_after_name": fields.Char("Clinic after name",required=False),
        "first_permanent_vascular_access": fields.Date("First time Permanent Vascular",required=False),
        "first_tenckhoff_catheters": fields.Date("First time Tenckhoff Catheters",required=False),
        "start_date_clinic": fields.Date("Start Date Clinic",required=False),
        "waiting_transplantation": fields.Selection([("yes","Yes"),("no","No")],"Kidney Transplantation Waiting ?"),
        "who_transplantation": fields.Char("Who is Transplantation?"),
        "reason_of_chronic_renal_failure": fields.Char("Reason chronic renal failure ?"),
        'causes': fields.One2Many("clinic.patient.cause","patient_id","Cause Line"),
        'comorbilities': fields.One2Many("clinic.patient.comorbidity","patient_id","Comorbility"),
        'morbilities': fields.One2Many("clinic.patient.morbidity","patient_id","Morbility"),
        "comments": fields.One2Many("message","related_id","Comments"),
        "company_id": fields.Many2One("company","Company",search=True),
        "visits": fields.One2Many("clinic.visit","patient_id","Visits"),
        "hd_cases": fields.One2Many("clinic.hd.case","patient_id","HD Cases"),
        "partner_id": fields.Many2One("partner","Contact"),
        "dialyzers": fields.One2Many("clinic.dialyzer","patient_id","Dialyzers"),
        "active":fields.Boolean("Uncheck as discountinue", search=True),
        'note': fields.Text("Note"),
        'categ_id': fields.Many2One("clinic.patient.categ","Category"),
        'doctor_id': fields.Many2One("clinic.staff","Doctor",domain=[['type','=','doctor']]),
        "documents": fields.One2Many("document","related_id","Documents"),
        'resign_date': fields.Date("Resign Date"),
        'rm_remain_visit': fields.Boolean("Auto Remove Remaining Visit"),
        'department_id': fields.Many2One("clinic.department","Department",search=True),
        'cycle_id': fields.Many2One("clinic.cycle","Last Cycle",function="_get_last_cycle"),
        'branch_id': fields.Many2One("clinic.branch","Branch",search=True),
        'cycles': fields.One2Many("clinic.patient.cycle","patient_id", "Cycles"),
        "vascular_acc": fields.Many2One("clinic.vascular.access","Vascular Ac."),
    }

    def _get_number(self,context={}):
        while 1:
            seq_id=get_model("sequence").find_sequence(name="Clinic Patient")
            num=get_model("sequence").get_next_number(seq_id,context=context)
            if not num:
                return None
            user_id=get_active_user()
            set_active_user(1)
            # reformat number: month-running num-year thai(2 digit)
            monthnow=int(time.strftime("%m"))
            yearnow=utils.date2thai(time.strftime("%Y-%m-%d"),"%(By)s")
            num="%s-%s-%s"%(monthnow,num,yearnow)

            res=self.search([["number","=",num]])
            set_active_user(user_id)
            if not res:
                return num
            get_model("sequence").increment_number(seq_id,context=context)
    
    def _get_cause_line(self,context={}):
        cause_ids=get_model("clinic.cause.chronic").search([])
        return cause_ids 
    
    def _get_type(self,context={}):
        ptype_id=None
        for ptype in get_model("clinic.patient.type").search_browse([]):
            if ptype.default:
                ptype_id=ptype.id
                break
        return ptype_id
    
    def _get_branch(self,context={}):
        b_ids=get_model('clinic.branch').search([])
        if b_ids:
            return b_ids[0]

    _defaults={
        "number": _get_number,
        "reg_date": lambda *a: time.strftime("%Y-%m-%d"),
        "company_id": lambda *a: get_active_company(),
        'branch_id': _get_branch,
        'card_type': 'identification',
        'type_id': _get_type,
        "active" : True,
    }
    _order="resign_date desc,number desc"

    def create(self, vals,**kw):
        obj_id=super(Patient,self).create(vals,**kw)
        obj=self.browse(obj_id)
        partner_id=obj.partner_id
        if not partner_id:
            for partner in get_model("partner").search_browse([['name', '=', obj.name]]):
                if partner.name==obj.name:
                    partner_id=partner.id
                    break
            if not partner_id:
                partner_id=get_model("partner").create({
                    'name': obj.name,
                    'last_name': obj.name,
                    'type': 'person',
                })
                address_id=get_model('address').create({
                    'type': 'shipping',
                    'partner_id': partner_id,
                    'patient_id': obj.id, #XXX
                    'address': 'your address',
                    'address2': 'your address2',
                    'city': 'your city',
                    'postal_code': 'your zip',
                    'country_id': 1,

                })
                get_model('address').browse(address_id).write({
                   'related_id': "clinic.patient,%s"%obj.id,
               })
            obj.write({
                'partner_id': partner_id,
            })
        return obj_id

    def delete(self,ids,context={}):
        partner_ids=[]
        for obj in self.browse(ids):
            partner_ids.append(obj.partner_id.id)
        address_ids=[addr.id for addr in obj.partner_id.addresses]
        get_model("partner").delete(partner_ids)
        get_model("address").delete(address_ids)
        super().delete(ids)

    def write(self,ids,vals,**kw):
        if 'active' in vals.keys():
            if not vals['active']:
                vals['resign_date']=time.strftime("%Y-%m-%d")

        for obj in self.browse(ids):
            if 'doctor_id' in vals.keys():
                for vs in get_model("clinic.visit").search_browse([['state','in',['draft','pending']],['patient_id','=',obj.id]]):
                    vs.write({
                        'doctor_id': vals['doctor_id'],
                    })
                    print("update doctor %s to patient %s"%(vals['doctor_id'], obj.name))
            #if not obj.addresses:
                #raise Exception("Address not found - %s!"%obj.name)
            partner_id=obj.partner_id
            if not partner_id:
                for partner in get_model("partner").search_browse([['name', '=', obj.name]]):
                    if partner.name==obj.name:
                        partner_id=partner.id
                        break
                if not partner_id:
                    partner_id=get_model("partner").create({
                        'name': obj.name,
                        'last_name': obj.name,
                        'type': 'person',
                    })
                vals['partner_id']=partner_id

            if not isinstance(partner_id,int):
                partner_id=partner_id.id 
            if obj.addresses:
                addr=obj.addresses[0]
                get_model("address").browse(addr.id).write({
                    'partner_id': partner_id,
                    'patient_id': obj.id,
                })
            else:
                if vals.get("addresses"):
                    addr_vals=vals.get("addresses")[0][1]
                    addr_id=get_model("address").create(addr_vals)
                    get_model("address").browse(addr_id).write({
                        'patient_id': obj.id,
                        'partner_id': partner_id,
                    })
                    del vals['addresses']
                    print("create address for %s"%obj.name, ' ', addr_vals)
            if obj.rm_remain_visit:
                visit_ids=get_model('clinic.visit').search([['patient_id','=',obj.id],['state','=','draft']])
                get_model('clinic.visit').delete(visit_ids)
                print('remove visit auto %s'%visit_ids)
        super().write(ids,vals,**kw)
    
    def name_get(self,ids,context={}):
        vals=[]
        for obj in self.browse(ids):
            name=obj.name or ''
            nick_name=obj.nick_name or ""
            if nick_name:
                name+=" (%s)"%nick_name
            if obj.hn_no:
                name+=" / %s"%obj.hn_no
            vals.append((obj.id,name,obj.image))
        return vals

    def name_search(self,name,domain=None,context={},**kw):
        dom=[["name","ilike","%"+name+"%"]]
        if domain:
             dom=[dom,domain] 
        ids1=self.search(dom)

        dom=[["hn_no","ilike","%"+name+"%"]]
        if domain:
            dom=[dom,domain]
        ids2=self.search(dom)

        ids=list(set(ids1+ids2))
        return self.name_get(ids,context=context)
    
    def simple_address(self,ids,context={}):
        print("call simple address ")
        for obj in self.browse(ids):
            if not obj.addresses:
                address_id=get_model("address").create({
                    'patient_id': obj.id,
                    'partner_id': obj.partner_id.id,
                    'type': 'shipping',
                    'address': 'xxxx',
                    'address2': 'xxxx',
                    'city': 'BKK',
                    'country_id': 1,
                    'postal_code': '12000',
                })
        if context.get('called'):
            return obj.id
        return {
            'next': {
                'name': 'clinic_patient',
                'mode': 'form',
                'active_id': obj.id,
            },'flash': 'Simple Address OK',
        }

    def new_dialyzer(self,ids,context={}):
        dlz_id=None
        for obj in self.browse(ids):
            dlz_vals=get_model("clinic.dialyzer").default_get()
            dlz_vals['patient_id']=obj.id
            dlz_vals['company_id']=dlz_vals['company_id'][0]
            dlz_vals['product_id']=dlz_vals['product_id'][0]
            dlz_id=get_model('clinic.dialyzer').create(dlz_vals)
            dialyzer=get_model("clinic.dialyzer").browse(dlz_id)
            dialyzer.confirm()
        if context.get('called'):
            return dlz_id
        return {
            'next': {
                'name': 'clinic_patient',
                'mode': 'form',
                'active_id': obj.id,
            },'flash': 'New Dialyzer successfully',
        }

Patient.register()
